function [U, M, model] = InfTuckerDP_vem(data, model, optimizer, uparam)

%% variable initialization
nvec = size(data.Y);
n = prod(nvec);
d = model.dim;
nmod = length(nvec);

% initialization
U = model.U;
M =  model.M; % data.Y; % 

%do initialization
model = do_init(U,model,nmod,nvec,'random',data);

iter_cut = ceil(model.em_total / 10);
for iter = 1:model.em_total
    
    if model.verbose && mod(iter,iter_cut) == 0
        fprintf('%g%% done, %d/%d iterations\n', ... 
            floor(iter/model.em_total*100), iter, model.em_total);
    end
    
    %variational E step
    kernel = ten_ker_func(U, uparam);
    if strcmp(model.loss, 'Gauss') || strcmp(model.loss, 'Probit')        
        [M, model] = var_inf(M, data.Y, kernel, model, U);
    end
    
    %M step
    tmp_Sigma0p5 = cell(nmod,1);
    inv_eigens = cell(nmod,1);
    for k = 1: nmod
       tmp_Sigma0p5{k} = kernel.V{k}* diag(double(kernel.D{k}).^(-0.5))*(kernel.V{k})';
       inv_eigens{k} = 1./kernel.D{k};
    end

    switch model.update_scheme
        case 1 % optimize U as a whole vector
            for k = 1: nmod % randperm(d) %
                ind_other = setdiff(1:nmod, k);     
                Utimsnok = ttm(M,tmp_Sigma0p5(ind_other'), ind_other');
                funObj = @(U_k)ten_gradU(U_k, k, Utimsnok, kernel.D,  kernel.V{k}, uparam, model);
                %U_knew = optimizer.algorithm(funObj,U{k}(:), model.lambda* ones(nvec(k)*d,1), optimizer.options);  
                U_knew = ncg(funObj, U{k}(:));
                U_knew = U_knew.X;
                U{k} = reshape(U_knew, nvec(k), d);
                kernel_k = ker_func(U{k}, uparam);
                kernel.V{k} = kernel_k.V;
                kernel.D{k} = kernel_k.D;
                tmp_Sigma0p5{k} = kernel.V{k}* diag(kernel.D{k}.^(-0.5))*(kernel.V{k})';
            end
        case 5 % stochastic gradient descent
            rate = model.rate;%1e-6;%0.5*1e-9;
            for k=1:nmod
                ind_other = setdiff(1:nmod, k);     
                Utimsnok = ttm(M,tmp_Sigma0p5(ind_other'), ind_other');
                funObj = @(U_k)ten_gradU(U_k, k, Utimsnok, kernel.D,  kernel.V{k}, uparam, model);
                [~, gk] = funObj(U{k}(:));
                U_knew = U{k}(:) - rate*gk;
                U{k} = reshape(U_knew, nvec(k), d);
                kernel_k = ker_func(U{k}, uparam);
                kernel.V{k} = kernel_k.V;
                kernel.D{k} = kernel_k.D;
                tmp_Sigma0p5{k} = kernel.V{k}* diag(kernel.D{k}.^(-0.5))*(kernel.V{k})';
            end
        otherwise
            error('unknown gradient update scheme');
    end

    % test on the holdout set
    if data.eval_verbose
         [val, name] = eval_performance(model, data, M);
         fprintf('iter = %d, InfTucker %s = %f\n', iter, name, val);
    end
end

% final result
kernel = ten_ker_func(U, uparam);
if strcmp(model.loss, 'Gauss') || strcmp(model.loss, 'Probit')
     M = var_inf(M, data.Y, kernel, model, U);
end
model.U = U;



