function [U, model] = InfTuckerDP_vem_online(data, model, uparam)

%% variable initialization
nvec = size(data.Y);
d = model.dim;
nmod = length(nvec);

% initialization
U = model.U;
M =  model.M; % data.Y; % 
%do initialization
if model.enable_dp
    model = do_init(U,model,nmod,nvec,'random',data);
end

iter_cut = 10;%ceil(model.em_total / 100);
data.nzind = find(data.Y);
for iter=1:model.stream_total
    if model.verbose && mod(iter,iter_cut) == 0
        fprintf('%d/%d subtensors \n', ... 
            iter, model.stream_total);
    end
    
    %sample a sub-tensor
    [subind, ~] = sub_sample(data.Y, nvec, model.subtensor_size, model);
    nzind = data.nzind;
    for k=1:nmod
        nzind = nzind( ismember(nzind(:,k),subind{k}),:);
        if isempty(nzind) 
            break; 
        end
    end
    if isempty(nzind)
        Y_sub = sptensor(model.subtensor_size);
        Y_sub = tensor(Y_sub);
    else
        rmap = cell(nmod,1);
        sub_ind = zeros(size(nzind));
        for j=1:nmod
            rmap{j} = zeros(max(subind{j}),1);
            rmap{j}(subind{j}) = 1:model.subtensor_size(j);
            sub_ind(:,j) = rmap{j}(nzind(:,j));
        end
        Y_sub = sptensor(sub_ind, data.Y(nzind), model.subtensor_size);
    end    
    U_sub = cell(length(U),1);
    for k=1:nmod
        U_sub{k} = U{k}(subind{k},:);
    end 
    if strcmp(model.loss, 'Gauss') || isempty(nzind)
        M_sub = Y_sub;
    else
        P = tucker_als(Y_sub, model.dim);
        M_sub = ttm(P.core,P.U); 
        M_sub = M_sub/norm(M_sub)*norm(Y_sub); %    
    end
    for k=1:nmod
        U_sub{k} = U{k}(subind{k},:);
    end
    
    %E-step to update local varaitional parameters q(M_sub)
    kernel = ten_ker_func(U_sub, uparam);
    M_sub = update_qM(M_sub, Y_sub, kernel, model);
    
    tmp_Sigma0p5 = cell(nmod,1);
    inv_eigens = cell(nmod,1);
    for k = 1: nmod
       tmp_Sigma0p5{k} = kernel.V{k}* diag(double(kernel.D{k}).^(-0.5))*(kernel.V{k})';
       inv_eigens{k} = 1./kernel.D{k};
    end
            
    %calculate the nosiy gradient for the subtensor
    g = cell(nmod,1);
    gsub = cell(nmod,1);
    Usub_old = cell(nmod,1);
    for k = 1: nmod 
        ind_other = setdiff(1:nmod, k);
        Utimsnok = ttm(M_sub,tmp_Sigma0p5(ind_other'), ind_other');
        [gsub{k}] = ten_stochastic_gradU(U_sub{k}(:), k, Utimsnok, kernel.D,  kernel.V{k}, uparam, model, model.stream_total); 
        if model.enable_dp
            g{k} = (U{k} - model.stat.dp_phi{k}*model.stat.eta_mean{k})*model.dp_lam(k);
            gsub{k} = g{k}(subind{k},:) + reshape(gsub{k},size(U_sub{k}));
        else
            gsub{k} = reshape(gsub{k},size(U_sub{k}));
        end
        %g{k}(subind{k},:) = gsub{k};
        %U{k} = U{k} - model.rate*g{k};
        
        U_sub{k} = U_sub{k} - model.rate*gsub{k};
        Usub_old{k} = U{k}(subind{k},:);
        U{k}(subind{k},:) = U_sub{k};
        %update gram matrix for the submode
        %U_sub{k} = U{k}(subind{k},:);
        kernel = ten_ker_func(U_sub, uparam);
        tmp_Sigma0p5{k} = kernel.V{k}* diag(double(kernel.D{k}).^(-0.5))*(kernel.V{k})';
    end
        
    if model.enable_dp
        %update global variational parameters q(v), q(eta), q(Z)
        if isfield(model, 'cache_stat')
            model = update_global_var_v2(nvec, model, U, Usub_old, subind);
        else
            model = update_global_var(nvec, model,U, subind);
        end
    end
    if mod(iter,20)==0 && data.eval_verbose      
            kernel = ten_ker_func(U, uparam);
            [Mtest] = update_qM(M, data.Y, kernel, model);
            [val, name] = eval_performance(model, data, Mtest);
            fprintf('iter = %d, InfTucker-DP %s = %f\n', iter, name, val);
    end
    
end










