%%regrading variational posterior and statistics
%model.T: 1*K vector, truncation number of each mode for variational inference of DP
%model.dp_lam: the inverse variance to generate latent factors under DP prior
%model.dp_lam_a: the parameter for posterior of model.dp_lam;
%model.dp_lam_b: the parameter for posterior of model.dp_lam;
%model.dp_lam_a0, the parameter for prior of model.dp_lam
%model.dp_lam_b0, the parameter for prior of model.dp_lam;
%model.dp_alpha: the beta prior to genreate v (for steak-breaking process)
%model.dp_ga{k}: T_k *2, for steak-breaking process construction
%model.dp_logphi, K*1 cell, dp_logphi{k} is a m_k * T_k matrix, n-th row is the

%model.stat.ex_logv E(log(v)), E(log(1-v)), K*1
%cell, ex_logv{k} is a T_k*2 vector, each column: E(log(v)), E(log(1-v))
%model.stat.dp_phi
%model.stat.eta_mean: K*1 cell, eta_mean{k} is a T_k*r_k matrix, t-th row
%is the mean of eta_k^t
%model.stat.eta_var: K*1 cell, eta_var{k} is T_k*1 vector, t-th element is
%E(|| eta_k^t ||^2)
%model.stat.eta_cov: K*1 cell, eta_cov{k} is T_k*1 vector, t-th element
%times I forms the covariance of eta_k^t

function model = do_init(U, model, nmod, nvec, strategy, data)
    %for variational posteriors/statistics    
    if ~isfield(model, 'T')
        model.T = round(nvec/10);    
    end
    model.dp_alpha = 1.0;
    model.dp_ga = cell(nmod,1);
    model.dp_logphi = cell(nmod,1);
    model.dp_lam_a0 = 1e-1*ones(1,nmod);
    model.dp_lam_b0 = 1e-3*ones(1,nmod);
    model.dp_lam_a = zeros(1, nmod);
    model.dp_lam_b = zeros(1,nmod);
    if model.update_inv_cov
        model.dp_lam = model.dp_lam_a0./model.dp_lam_b0;
    end
    model.stat.ex_logv = cell(nmod,1);
    model.stat.dp_phi = cell(nmod,1);
    model.stat.eta_mean = cell(nmod,1);
    model.stat.eta_var = cell(nmod,1);
    model.stat.eta_cov = cell(nmod,1);
    for k=1:nmod
        trunc_no = model.T(k);
        model.dp_ga{k} = zeros(trunc_no,2);
        model.dp_logphi{k} = zeros(nvec(k),trunc_no);
        model.stat.ex_logv{k} = zeros(trunc_no,2);
        model.stat.eta_mean{k} = zeros(trunc_no,model.dim);
        model.stat.eta_var{k} = zeros(1,trunc_no);
        model.stat.eta_cov{k} = zeros(1,trunc_no);
    end
    if strcmp(strategy, 'U-specific')
        %use gmm to obtain centers from U, q(Z), q(eta)
        for k=1:nmod
            [~, model.stat.dp_phi{k}, gmm_model] = emgm(U{k}',model.T(k));
            model.stat.eta_mean{k} = gmm_model.mu';
            model.stat.eta_cov{k} = ones(1, model.T(k));
            model.stat.eta_var{k} = model.dim + sum(model.stat.eta_mean{k}.^2,2)';
        end
    else if strcmp(strategy, 'random')
            %complete random
            for k=1:nmod
                dirpara = ones(1, model.T(k));                
                model.stat.dp_phi{k} = drchrnd(dirpara,nvec(k));
                %model.stat.dp_phi{k} = 1/model.T(k)*ones(nvec(k),model.T(k));
                model.stat.eta_mean{k} = randn(model.T(k), model.dim);
                model.stat.eta_cov{k} = ones(1, model.T(k));
                model.stat.eta_var{k} = model.dim + sum(model.stat.eta_mean{k}.^2,2)';
            end
        end        
    end
    
    %create the cache stats
    if isfield(model, 'cache_stat')
        model.cache_stat = [];
        %\sum \Phi_k^{n,t}
        model.cache_stat.psy1 = cell(nmod,1);
        %\sum \Phi_k^{n,t}U_k^n
        model.cache_stat.psy2 = cell(nmod,1);
        %\sum_{n=1}^m_k \sum_{j=t+1}^T \Phi_k^{n,j}
        model.cache_stat.psy3 = cell(nmod,1);
        for k=1:nmod
            model.cache_stat.psy1{k} = sum(model.stat.dp_phi{k}, 1);
            model.cache_stat.psy2{k} = model.stat.dp_phi{k}'*U{k};
            model.cache_stat.psy3{k} = zeros(1,model.T(k));
            csum = cumsum(fliplr(model.cache_stat.psy1{k}));
            csum = fliplr(csum(1:end-1));
            model.cache_stat.psy3{k} = [csum,0];                                
        end
    end
    
    %for sampling in online algorithm
    if isfield(model,'sample_strategy')
        switch model.sample_strategy
            case 'weighted'
                nvec = size(data.Y);
                nmod = length(nvec);
                model.weight = cell(nmod, 1);
                for k = 1:nmod
                    model.weight{k} = histc(data.train(:,k), 1:nvec(k)) + 1;
                end
            case 'weighted2'
                nvec = size(data.Y);
                nmod = length(nvec);
                model.weight = cell(nmod, 1);
                for k = 1:nmod
                    model.weight{k} = log(histc(data.train(:,k), 1:nvec(k)) + 2);
                end    
            case 'grid'
                nvec = size(data.Y);
                nmod = length(nvec);
                repeat = 5;
                model.partition = cell(repeat,1);
                model.par_num = cell(nmod,1);
                for j = 1:repeat
                    model.partition{j} = cell(nmod,1);
                    for k = 1:nmod
                        num = floor(nvec(k)/model.subtensor_size(k));
                        model.par_num{k} = num;
                        model.partition{j}{k} = crossvalind('Kfold',nvec(k), num);
                    end
                end
        end
    end
    
end