function kernel = ker_func(X, param)
% kernel function with a minimal design

kernel = param;
n = size(X, 1);

switch param.type
    case 'linear'
        kernel.K = X*X' + param.kreg * eye(n);
    case 'rbf'
        P=sum(X.*X,2);
        kernel.K = param.weight* exp(-param.gamma*(repmat(P',n,1) + repmat(P,1,n) ... 
            - 2*(X*X'))) + param.kreg * eye(n);
    case 'exp'
        P=sum(X.*X,2);
        kernel.K = param.weight* exp(-param.gamma*(repmat(P',n,1) + repmat(P,1,n) ... 
            - 2*(X*X')).^0.5) + param.kreg * eye(n);
    case 'poly'
        kernel.K = (X*X').^param.degree + param.kreg * eye(n);
    case 'tapcov'
        kernel.K = taperCov(X, param) + param.kreg * eye(n);
    otherwise
        error('Unknown kernel type');
end

%% Eigen decomposition
[V, D] = eig(full(kernel.K)); %eig(kernel.K);
dD = diag(D);
%mdD = max(dD);
%rtol = param.tol * mdD;
%kernel.D = dD(dD > rtol);
%kernel.V = V(:, dD > rtol);
kernel.D = dD;
kernel.V = V;
kernel.diagonal = diag(full(kernel.K));