function [Ugrad ]= taperCovgrad(U, uparam)

% parameters of 
nu = uparam.nu;
alpha = uparam.alpha;
phi = uparam.phi;
theta = uparam.theta;
threshold = uparam.decision;

[n dim] = size(U);
% Ugrad = sparse(n,n);
Ugrad = cell(dim,1);

% calculate the gradient of matern covariance
%% distance function
switch uparam.distance
    case 'Euclidean'
        if ~uparam.savestorage
            P=sum(U.*U,2);
            H = sqrt(repmat(P',n,1) + repmat(P,1,n) - 2*(U*U'));
%             [Hindi, Hindj] = find( H< theta);
            Hvec = reshape(H, n*n,1);
            [Hindi, Hindj] = find( Hvec< theta);
            Hvec = Hvec(Hindi);
        else
        % second choice: to save storage space, only cases of H<theta
            nzmax = n*n;
            Hvec = sparse(nzmax,1);
            Hindi = sparse(nzmax,1);
            Hindj = sparse(nzmax,1);
            curlen = 0;
            for i=1: n
                tmp =  sqrt( sum((repmat(U(i,:),n,1) - U).^2, 2) );
                [indi, indj] = find( tmp< theta);
                len = length(indi);
                Hindi(curlen+1:  curlen+len) = i*ones(len,1);
                Hindj(curlen+1:  curlen+len) = indi;
                Hvec(curlen+1:  curlen+len) = tmp(indi);
                curlen = curlen+ len;
            end
            if curlen <= nzmax
                Hindi = full(Hindi(1:curlen,1));
                Hindj = full(Hindj(1:curlen,1));
                Hvec = full(Hvec(1:curlen,1));
            end
        end
%     case 'others'
%         break;
end
Hvec = Hvec.* (abs(Hvec)> threshold);
%Hvec = Hvec + threshold; %zsd

%% taper function used
trunc = (1- Hvec/theta).*(Hvec < theta );
switch uparam.taper
    case 'Spherical'    
        CovTapvec = trunc.^2 .*  (1 + Hvec/(2*theta));
        CovTapvec_grad = (-2/theta)*trunc.* (1 + Hvec/(2*theta))+ trunc.^2 /(2*theta);
    case 'Wendland1'
        CovTapvec = ((1- Hvec/theta).*( Hvec < theta )).^4.*  (1 + 4*Hvec/theta); 
    case 'Wendland2'
        CovTapvec = ((1- Hvec/theta).*( Hvec < theta )).^6.* (1 + 6*Hvec/theta + 35*Hvec.^2/3/(theta.^2) ); 
end


%% calculate 
% if ~uparam.savestorage
%     CovTapvec = CovTapvec .* (CovTapvec > threshold);
%     [ind, indj] = find(CovTapvec);
%     Sigmavec = phi/2^(nu-1)/gamma(nu)* (alpha*Hvec(ind)).^nu.* ...
%         besselk(nu,alpha*Hvec(ind)).* CovTapvec(ind);
%     % sparse representation
%     indrow = fix(ind/ n) +1;
%     indcol = rem(ind, n); 
%     indcol = indcol + n*(indcol==0);
% 
%     Sigma = sparse(indrow, indcol, Sigmavec);
%     Sigma = Sigma .* (~isinf(Sigma));
%     %% continue here
% else
    Sigmavec = phi/2^(nu-1)/gamma(nu)* (alpha*Hvec).^nu.*  besselk(nu,alpha*Hvec);
    ind_nan = isnan(Sigmavec)==1;
    Sigmavec(ind_nan) =  1;
    %tmp = (besselk(nu-1,alpha*Hvec) - besselk(nu+1,alpha*Hvec)); %zsd: wrong: this is derivative for the Bessel function of second kind (not modified Bessel function of second kind), it also lacks of coefficient 0.5
    tmp = -0.5*(besselk(nu-1,alpha*Hvec) + besselk(nu+1,alpha*Hvec)); tmp(isinf(tmp))=1;%zsd, the correct derivative
    ind_nan = find(isnan(tmp));
    tmp(ind_nan) =  1;
    Sigmavec_grad = phi/2^(nu-1)/gamma(nu)*( nu*(alpha*Hvec).^(nu-1).* besselk(nu,alpha*Hvec) ...
        + (alpha*Hvec).^nu.*tmp); %zsd: wrong, lack of alpha as coefficient
    Sigmavec_grad(ind_nan) =0;
    Sigmavec_grad(isinf(Sigmavec_grad)) = 0;
    grad = Sigmavec.* CovTapvec_grad + Sigmavec_grad.*CovTapvec; 
    grad = sparse(Hindi, Hindj, grad);
    grad = reshape(grad,n,n);
    
    
    for i=1:dim
        tmp = (repmat(U(:,i),1,n) - repmat((U(:,i))',n,1))./(H + threshold);
%         Ugrad(:,i) = sum(sparse(Hindi, Hindj,  tmp).*grad, 1);
        Ugrad{i}=  tmp.*grad;
    end
end