function [U M mse] = tentp_gauss(data, model, optimizer, uparam)

%% variable initialization
nvec = size(data.Y);
n = prod(nvec);
d = model.dim;
nmod = length(nvec);

% initialization
U = model.U;
M = data.Y;

% %% decide the truncation parameter theta for tapering
% if strcmp(uparam.type, 'tapcov')
%     uparam.theta = (max(data.Y.vals)+min(data.Y.vals))/2
% end

iter_cut = ceil(model.em_total / 10);

for iter = 1:model.em_total
    
    if model.verbose && mod(iter,iter_cut) == 0
        fprintf('%g%% done, %d/%d iterations\n', ... 
            floor(iter/model.em_total*100), iter, model.em_total);
    end
    
%     % E-step
    kernel = ten_ker_func(U, uparam);
   
    tmp_Sigma0p5 = cell(nmod,1);
    for k = 1: nmod
       tmp_Sigma0p5{k} = kernel.V{k}* diag(double(kernel.D{k}).^(-0.5))*(kernel.V{k})';
    end
    
    if strcmp(model.process, 'TP')
        beta1 = (uparam.delta + n) /2;
        MtKM = ttm(M,tmp_Sigma0p5, 1:nmod);
        DD = tensor(ktensor(kernel.D));
        DD =  DD./(uparam.tau+DD);
        beta2 = (uparam.delta +  norm(MtKM)^2 + ttv(DD,kernel.D) )/2;
        uparam.tau = beta1/beta2;
    end
    
    % M-step
    switch model.update_scheme
        case 1 % optimize U as a whole vector
            for k = 1: nmod % randperm(d) %
                ind_other = setdiff(1:nmod, k);     
                Utimsnok = ttm(M,tmp_Sigma0p5(ind_other'), ind_other');
                funObj = @(U_k)ten_gradU(U_k, k, Utimsnok, kernel.D,  kernel.V{k}, uparam, model.process);
                
                % choice 1
                U_knew = optimizer.algorithm(funObj,U{k}(:), model.lambda * ones(nvec(k)*d,1), optimizer.options);
                % choice 2
%                 U_knew = optimizer.algorithm(funObj,U{k}(:), optimizer.options);
                
                U{k} = reshape(U_knew, nvec(k), d);
                kernel_k = ker_func(U{k}, uparam);
                kernel.V{k} = kernel_k.V;
                kernel.D{k} = kernel_k.D;
                tmp_Sigma0p5{k} = kernel.V{k}* diag(kernel.D{k}.^(-0.5))*(kernel.V{k})';
            end
        case 2 % column-wise optimization
            for k = randperm(d)
                funObj = @(w)gradU2(w, M, kernel, k);
                Uk = optimizer.algorithm(funObj,U(:,k), ...
                    model.lambda * ones(n, 1), optimizer.options);
                U(:,k) = Uk;
            end
        case 3 % l_1/2 regularizer
            funObj = @(U)gradU3(U, M, kernel, model);
            U = optimizer.algorithm(funObj,U(:), ...
                model.lambda * ones(n * d, 1), optimizer.options);
            U = reshape(U, n, d);
        case 4 % column-wise updating
            for k= 1: d % randperm(d) %
              ukvec = U(:,k);
              funObj = @(x)gradU_col(x, k, U, M, kernel, uparam);
%             ukvec = optimizer.algorithm(funObj, ukvec, optimizer.options);
              ukvec = optimizer.algorithm(funObj, ukvec, model.lambda * ones(n, 1),  optimizer.options);
              U(:,k)= ukvec;
            end
            U = reshape((U), n, d);
        otherwise
            error('unknown gradient update scheme');
    end
end

% final AUC
kernel = ten_ker_func(U, uparam);
M = optX_tentp_var([], data.Y, kernel, data.beta, model, uparam);
% M is a mixture of inference and known values
% alpha = 0.5;
% M = alpha*M + (1-alpha)* data.Y;

% M(data.subs)=0;
% M= sptensor(M);
%% ???? problem in the following
% M = sptensor(ttm(M, kernel.K, 1:nmod));
mse = mean((data.Ymiss- M(data.subs)).^2);

