%model.T: 1*K vector, truncation number of each mode for variational inference of DP
%model.dp_lam: the inverse variance to generate latent factors under DP
%prior over each mode
%model.dp_alpha: the beta prior to genreate v (for steak-breaking process)
%model.dp_ga{k}: T_k *2, for steak-breaking process construction
%model.dp_logphi, K*1 cell, dp_logphi{k} is a m_k * T_k matrix, n-th row is the
%varitional dist. for the selecting variable for n-th node in k-th mode
%k-th mode; dp_eta{k} is a T_k*r_k matrix, t-th row is the t-th cluster center for
%k-th mode
%variance for variational posterior of q(eta_k^t)

%model.stat, sub-field to save the statistics
%model.stat.ex_logv E(log(v)), E(log(1-v)), K*1
%cell, ex_logv{k} is a T_k*2 vector, each column: E(log(v)), E(log(1-v))
%model.stat.dp_phi
%model.stat.eta_mean: K*1 cell, eta_mean{k} is a T_k*r_k matrix, t-th row
%is the mean of eta_k^t
%model.stat.eta_var: K*1 cell, eta_var{k} is T_k*1 vector, t-th element is
%E(|| eta_k^t ||^2)
%model.stat.eta_cov: K*1 cell, eta_cov{k} is T_k*1 vector, t-th element
%times I forms the covariance of eta_k^t
function [M, model] = var_inf(M, Y, kernel, model, U)
% M_init, Y: tensor
% kernel.V, kernel.D: cell
tol = 1e-10;
nvec = size(Y);
nmod = length(nvec);
var_total = model.var_total;
loss = model.loss;
sigma2 = model.sigma2;

for iter = 1:model.var_total
    %update q(eta), the cluster centers sampled from base measure
    for k=1:nmod    
        model.stat.eta_cov{k} = 1./(1 + model.dp_lam(k)*sum(model.stat.dp_phi{k},1));
        model.stat.eta_mean{k} = model.stat.dp_phi{k}'*U{k}./(1/model.dp_lam(k) + repmat(sum(model.stat.dp_phi{k},1)',1,size(U{k},2)) );
        model.stat.eta_var{k} = model.dim*model.stat.eta_cov{k} + sum(model.stat.eta_mean{k}.^2,2)';
    end
        
    %update q(Z), indicators sampled from stick-breaking process
    for k=1:nmod
        trunc_no = model.T(k);
        sum_t = zeros(1,trunc_no);
        ex_logv2 = model.stat.ex_logv{k}(:,2);
        for t=2:trunc_no
            sum_t(t) = sum_t(t-1) + ex_logv2(t-1);
        end
        for n=1:nvec(k)
            model.dp_logphi{k}(n,:) = model.stat.ex_logv{k}(:,1)' + sum_t - 0.5*model.dp_lam(k)*model.stat.eta_var{k} ...
                + model.dp_lam(k)*U{k}(n,:)*model.stat.eta_mean{k}';
            model.stat.dp_phi{k}(n,:) = exp_sum_dist(model.dp_logphi{k}(n,:));
        end
    end
    
    %update q(V), for stick-breaking process construction
    for k=1:nmod
        trunc_no = model.T(k);
        model.dp_ga{k}(:,1) = 1 + sum(model.stat.dp_phi{k}, 1)';    
        for t=1:trunc_no
            model.dp_ga{k}(t,2) = model.dp_alpha + sum(sum(model.stat.dp_phi{k}(:,t+1:end), 1));
        end
        disum = psi(sum(model.dp_ga{k},2));
        model.stat.ex_logv{k} = [psi(model.dp_ga{k}(:,1)) - disum, psi(model.dp_ga{k}(:,2)) - disum];    
    end
    
    %update q(lam), for inverse variance of DP generating latent factors
    if model.update_inv_cov
        for k=1:nmod
            model.dp_lam_a(k) = model.dp_lam_a0(k) + 0.5*nvec(k)*model.dim;
            model.dp_lam_b(k) = model.dp_lam_b0(k) + 0.5*(trace(U{k}*U{k}') + sum(model.stat.dp_phi{k}*model.stat.eta_var{k}')) ...
                                - sum(sum(model.stat.eta_mean{k}*U{k}'.*model.stat.dp_phi{k}'));
            model.dp_lam(k) = model.dp_lam_a(k)/model.dp_lam_b(k);                
        end
    end
        
end
%update q(M)
DD = tensor(ktensor(kernel.D));
tau = 1;

if strcmp(loss, 'Gauss') 
    DD =  DD ./ (tau*sigma2 + DD);
else
    DD = DD ./ (tau + DD);
end
Vt = cell(nmod,1);
for k=1: nmod
    Vt{k} = (kernel.V{k})';
end

if strcmp(loss, 'Gauss') 
    M = tensor(ttm( ttm(Y,Vt,1:nmod).*DD, kernel.V, 1:nmod));
else %binary observation, using probit model
    beta = 2;
    for k = 1:var_total;
        P = (2*Y-1) .* tenfun(@(x) normpdf(x-beta), M ) ... %elemfun(M, @(x) normpdf(x-beta)) 
            ./ tenfun(@normcdf, 2*Y.*(M - beta) -M+beta  + tol );
        A = ttm(M+P,Vt,1:nmod).*DD;
        M = ttm( A, kernel.V, 1:nmod);        
    end
end
